@@ -26,6 +26,7 @@
 #include <stdlib.h>
 #include <time.h>
 #include <unistd.h>
+#include <sys/types.h>
 
 #include <glib.h>
 #include <glib/gi18n.h>
@@ -71,12 +72,21 @@ main (int argc, char **argv)
   const char *home_dir;
   GError *error = NULL;
 
+  if (G_UNLIKELY (getuid () != geteuid () ||
+                  getgid () != getegid () ||
+                  geteuid () == 0 || 
+                  getegid () == 0)) {
+    g_printerr ("Wrong euid/egid, exiting.\n");
+    return EXIT_FAILURE;
+  }
+
   setlocale (LC_ALL, "");
 
   terminal_i18n_init (TRUE);
 
   if (!g_get_charset (NULL)) {
     g_printerr ("Non UTF-8 locale is not supported!\n");
+    return EXIT_FAILURE;
   }
 
 #ifndef ENABLE_DISTRO_PACKAGING
