@@ -32,7 +32,8 @@ typedef enum {
   TERMINAL_DEBUG_MDI           = 1 << 4,
   TERMINAL_DEBUG_PROCESSES     = 1 << 5,
   TERMINAL_DEBUG_PROFILE       = 1 << 6,
-  TERMINAL_DEBUG_SETTINGS_LIST = 1 << 7
+  TERMINAL_DEBUG_SETTINGS_LIST = 1 << 7,
+  TERMINAL_DEBUG_APPMENU       = 1 << 8
 } TerminalDebugFlags;
 
 void _terminal_debug_init(void);
